package com.ibm.uia.renderer;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import java.util.*;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import com.ibm.uia.renderingInterface.*;

/**
 * This type was created in VisualAge.
 */
public class RendererTreeNode extends RendererObjectNode {
	// half the length of the sides of the hotspot rectangle
	private static final int HOTSPOTSIZE = 4;
	// distance between (centre of hotspot) and (right hand end of line to text)
	private static final int LINELENGTH = 10;

	// this is the list of child nodes IN ORDER
	private Vector children = new Vector();
	// the parent of the node (could be null)
	private RendererTreeNode parent;
	
	/**
	 * remember the state of expansion.
	 * true = children visible
	 * false = children hiden
	 */
	private boolean expanded = false;
	/**
	 * The centre of the rectangle
	 * bounding the expand/collapse symbol
	 */
	private Point hotspot;

public RendererTreeNode(RenderableObject renderableObject, RendererTreeNode parent) {
	super(renderableObject);
	this.parent = parent;
	hotspot = new Point();
	
	recalculateChildren(true);
}
/**
 * calculate maximum height based on all heights.
 */
protected int _calculateMaxHeight() {
	int maxHeight = super._calculateMaxHeight();
	if (HOTSPOTSIZE * 2 > maxHeight) {
		maxHeight = HOTSPOTSIZE * 2;
	}
	return maxHeight;
}
/**
 *
 */
protected void _drawAttachmentLines(Graphics g) {
	Color oldColour = g.getColor();
	g.setColor(Color.lightGray.darker());
	g.drawLine(hotspot.x, hotspot.y, hotspot.x + HOTSPOTSIZE + LINELENGTH, hotspot.y);
	if (null != parent) {
		Point parentAttachPoint = parent.getChildLineAttachmentPoint();
		g.drawLine(hotspot.x, parentAttachPoint.y, hotspot.x, hotspot.y);
	}
	g.setColor(oldColour);
}
/**
 * draw the hotspot onto the graphics provided.
 * we do this after all the receivers siblings have
 * been drawn, because the algorithm is simpler. 
 */
void _drawHotspot(Graphics g) {
	if (!children.isEmpty()) {
		int x1 = hotspot.x - HOTSPOTSIZE;
		int x2 = hotspot.x + HOTSPOTSIZE;
		int y1 = hotspot.y - HOTSPOTSIZE;
		int y2 = hotspot.y + HOTSPOTSIZE;

		Color oldColour = g.getColor();
		g.setColor(Color.white);
		g.fillRect(x1,y1,2*HOTSPOTSIZE,2*HOTSPOTSIZE);
		g.setColor(Color.lightGray.darker());
		g.drawRect(x1,y1,2*HOTSPOTSIZE,2*HOTSPOTSIZE);
		x1 +=2;
		x2 -=2;
		g.drawLine(x1, hotspot.y, x2, hotspot.y);
		if (!expanded) {
			y1 +=2;
			y2 -=2;
			g.drawLine(hotspot.x, y1, hotspot.x, y2);
		}
		g.setColor(oldColour);
	}	
}
private final RendererTreeNode _getNodeForObject(RenderableObject ro) {
	Enumeration enum = children.elements();
	while (enum.hasMoreElements()) {
		RendererTreeNode rtn = (RendererTreeNode)enum.nextElement();
		if (rtn.getRenderableObject().equals(ro)) {
			return rtn;
		}
	}
	return null;
}
private final boolean _hasChildren() {
	return children.size() > 0;
}
/**
 * reply the x coordinate of the left hand side of the
 * image
 */
protected int _imageLeft(boolean isTree) {
	if (!isTree) {
		return super._imageLeft(isTree);
	}
	if (null != parent) {
		Point pt = parent.getChildLineAttachmentPoint();
		hotspot.x = pt.x;
	} else {
		hotspot.x = HOTSPOTSIZE;
	}	
	return hotspot.x + HOTSPOTSIZE + LINELENGTH;
}
void addVisibleChildrenTo(Vector v) {
	v.addElement(this);
	if (expanded) {
		Enumeration enum = children.elements();
		while (enum.hasMoreElements()) {
			((RendererNode)enum.nextElement()).addVisibleChildrenTo(v);
		}
	}
}
/**
 * draw the receiver onto the graphics provided.
 * return the y coordinate of the bottom of the last element drawn.
 */
public int draw(Graphics g, int y, boolean isTree, ImageObserver refreshPanel) {
	int bottom = super.draw(g, y, isTree, refreshPanel);

	if (isTree) {
		// now we can place the hotspot	
		hotspot.y = (y + bottom) / 2;
		_drawAttachmentLines(g);
		if (expanded) {
			Enumeration enum = children.elements();
			while (enum.hasMoreElements()) {
				RendererTreeNode mtn = (RendererTreeNode) enum.nextElement();
				bottom = mtn.draw(g, bottom, isTree, refreshPanel);
			}
			// draw all child hotspots after all their attachment lines - then
			// we can simply stamp over the lines with th hospot, and do not have
			// to calculate intersections and such.
			enum = children.elements();
			while (enum.hasMoreElements()) {
				RendererTreeNode mtn = (RendererTreeNode) enum.nextElement();
				mtn._drawHotspot(g);
			}
		}
		if (null == parent) {
			_drawHotspot(g);
		}
	}
	return bottom;
}
/**
 * find the first visible node whose yPosition
 * is greater than the y coordinate of the given
 * point, and return it.
 */
RendererNode find(Point pt) {
	RendererNode rn = super.find(pt);
	if (null != rn) {
		return rn;
	}
	if (expanded) {
		Enumeration enum = children.elements();
		while (enum.hasMoreElements()) {
			RendererNode rtn = ((RendererNode)enum.nextElement()).find(pt);
			if (null != rtn) {
				return rtn;
			}
		}
	}
	return null; 
}
protected Point getChildLineAttachmentPoint() {
	Rectangle imageRect = getImageRect();
	return new Point(
		imageRect.x + (imageRect.width /2),
		imageRect.y + imageRect.height);
}
Vector getChildNodeVector() {
	return children;
}
public final boolean getExpanded() {
	return expanded;
}
RendererTreeNode getParent() {
	return parent;
}
int getWidth() {
	int max = super.getWidth();
	Enumeration enum = children.elements();
	while (enum.hasMoreElements()) {
		RendererNode rn = (RendererNode)enum.nextElement();
		int i = rn.getWidth();
		if (i > max) {
			max = i;
		}
	}
	return max;
}
/**
 * reply true if the point (described by the given cordinate)
 * lies within the hotspot area
 */
public final boolean isHotspot(Point pt) {
	return 	pt.x >= hotspot.x - HOTSPOTSIZE
				&& pt.x <= hotspot.x + HOTSPOTSIZE
				&& pt.y >= hotspot.y - HOTSPOTSIZE
				&& pt.y <= hotspot.y + HOTSPOTSIZE;
}
public boolean isRoot() {
	return false;
}
/**
 * Reply true if the receiver, or one of the children (recursive
 * definition) is selected
 */
public final boolean isSelectedOrChildSelected() {
	if (selected) {
		return true;
	}
	Enumeration enum = children.elements();
	while (enum.hasMoreElements()) {
		RendererTreeNode rtn = (RendererTreeNode)enum.nextElement();
		if (rtn.isSelectedOrChildSelected()) {
			return true;
		}
	}
	return false;
}
/**
 * create and return an initialised instance of RendererTreeNode
 * for the receiver to use as a child.
 * This has been exposed to subclasses so that they can override
 * and substitute their own tree nodes.
 */
protected RendererTreeNode newChildNode(RenderableObject rob) {
	return new RendererTreeNode(rob, this);
}
RendererNode nodeAtIndex(int i) {
	if (0 == i) {
		return this;
	} else if (expanded) {
		Enumeration enum = children.elements();
		while (enum.hasMoreElements()) {
			i = i - 1;
			RendererNode child = (RendererNode)enum.nextElement();
			RendererNode found = child.nodeAtIndex(i);
			if (null != found) {
				return found;
			}
		}
	}
	return null;
}
public void recalculate(boolean recurse) {
	recalculateChildren(recurse);
}
final void recalculateChildren(boolean recurse) {
	// create a new vector of child nodes, as the Vector class
	// will be scanned for matches
	Vector newChildren = new Vector();
	Enumeration objectEnum = renderableObject.getAllChildren();
	while (objectEnum.hasMoreElements()) {
		// itterate through real children of object
		RenderableObject ro = (RenderableObject)objectEnum.nextElement();
		// find matching node
		RendererTreeNode rtn = _getNodeForObject(ro);
		if (null == rtn) {
			// create a matching node
			rtn = newChildNode(ro);
		}
		newChildren.addElement(rtn);
		if (recurse) {
			rtn.recalculateChildren(recurse);
		}
	}
	children = newChildren;
}
public final void setExpanded(boolean set) {
	expanded = set;
}
}
